'use strict';

angular.module('spmApp')

    .config(function ($compileProvider) {
        $compileProvider.aHrefSanitizationWhitelist(/^\s*(http|https|mailto|tel|maps|geo):/);
    })

    /** LOGGING **/
    .config([
        '$logProvider', 'DEBUG',
        function ($logProvider, DEBUG) {
            $logProvider.debugEnabled(DEBUG);
        }
    ])

    /** ROUTING **/
    .config([
        '$routeProvider', 'PREMIUM',
        function ($routeProvider, PREMIUM) {
            $routeProvider.when('/search', {
                templateUrl: 'app/partials/search.html',
                controller: 'SearchController'
            });
            if (PREMIUM) {
                $routeProvider.when('/favs', {
                    templateUrl: 'app/partials/favoriten.html',
                    controller: 'FavoriteController'
                });
            }
            $routeProvider.when('/rss', {
                templateUrl: 'app/partials/rss.html',
                controller: 'RssController'
            });
            $routeProvider.when('/settings', {
                templateUrl: 'app/partials/settings.html',
                controller: 'SettingsCtrl'
            });
            $routeProvider.when('/impressum', {
                templateUrl: 'app/partials/impressum.html',
                controller: 'StandardController'
            });
            $routeProvider.when('/debug', {
                templateUrl: 'app/partials/debug.html',
                controller: 'DebugController'
            });
            $routeProvider.otherwise({redirectTo: '/search'});
        }
    ])

    .run([
        'PREMIUM', 'MARKENLOGOS', '$rootScope',
        function (PREMIUM, MARKENLOGOS, $rootScope) {
            $rootScope.isPremium = function () {
                return PREMIUM;
            };
            $rootScope.showMarkenlogos = function () {
                return MARKENLOGOS;
            };
        }
    ])

    .run([
        '$platform', 'settings', '$timeout', '$rootScope', '$cordovaDialogs',
        function ($platform, settingService, $timeout, $rootScope, $cordovaDialogs) {
            $platform.ready(function () {
                $timeout(function () {
                    settingService.get(function (settings) {
                        var search = settings.getSearch();

                        if (search.firstrun === undefined) {
                            var message = 'Möchten Sie zu Ihrem aktuellen Standort immer die Spritpreise angezeigt bekommen? '
                                + 'Diese Option können Sie jederzeit in den Einstellungen ändern.';

                            $cordovaDialogs.confirm(message, 'Standortbestimmung', ['Ja', 'Nein']) // Ja === 1 ; Nein === 2
                                .then(function (button) {
                                    search.firstrun = false;
                                    search.standort = (button === 1);

                                    settingService.update(search, function (settings) {
                                        $rootScope.$broadcast('settings.updated', settings);
                                    });
                                });
                        }
                    });
                });
            });
        }
    ])

    /** Banner **/
    .run([
        '$rootScope', '$platform', '$timeout', 'BannerService', 'PREMIUM',
        function ($rootScope, $platform, $timeout, BannerService, PREMIUM) {
            if (true === PREMIUM) {
                return;
            }

            $platform.ready(function () {
                $timeout(function () {
                    BannerService.retrieveBannerImg().then(function (value) {
                        $rootScope.$broadcast('banner.top.changed', value);
                    });
                });
            });
        }
    ]);
