'use strict';

angular.module('spmApp.controllers')
    .controller('FavoriteController', [
        '_', '$scope', 'Favorites', 'Monitor', 'settings', '$route', '$templateCache',
        function (_, $scope, Favorites, Monitor, settings, $route, $templateCache) {
            $scope.passwordUri = Favorites.getPasswordUri();
            $scope.registerUri = Favorites.getRegisterUri();
            $scope.sorten = Monitor.getSorten();
            $scope.favorites = {};
            $scope.user = '';
            $scope.password = '';
            $scope.sorte = '';
            $scope.authenticated = false;
            $scope.error = false;
            $scope.loading = false;

            $scope.fetchFavorites = function (sorte) {
                $scope.loading = true;
                if (sorte) {
                    $scope.sorte = sorte.value;
                }

                Favorites.getBookmarks($scope.user).then(function (data) {
                    Monitor.getTankstellenByIds(data, $scope.sorte)
                        .then(function (tankstellen) {
                            $scope.favorites = tankstellen;
                            $scope.loading = false;
                        });
                }, function () {
                    $scope.loading = false;
                });
            };

            $scope.authenticate = function (user, password) {
                Favorites.authenticate(user, password).then(function (user) {
                    var currentPageTemplate = $route.current.templateUrl;
                    $templateCache.remove(currentPageTemplate);
                    $route.reload();
                }, function () {
                    $scope.user = '';
                    $scope.password = '';
                    $scope.authenticated = false;
                    $scope.error = true;
                    $scope.favorites = {};
                });
            };

            $scope.logout = function () {
                Favorites.logout().then(function () {
                    $scope.user = '';
                    $scope.authenticated = false;
                    $scope.favorites = {};
                });
            };

            $scope.isSorteActive = function (sorte) {
                if ($scope.sorte) {
                    return $scope.sorte === sorte.value;
                }
                return false;
            };

            $scope.hasElements = function (array) {
                if (_.isArray(array)) {
                    return _.size(array) > 0;
                }
                return false;
            };

            settings.get(function (settings) {
                var search = settings.getSearch();
                $scope.sorte = search.sorte.value;
            });

            Favorites.isAuthenticated().then(function (user) {
                $scope.user = user;
                $scope.authenticated = true;
                $scope.fetchFavorites();
            });
        }
    ]);