'use strict';

var controllers = angular.module('spmApp.controllers', ['phonegap', 'logger', 'analytics']);

controllers.controller('StandardController', [
    '$scope', '$location', 'RSS',
    function ($scope, $location, RSS) {
        $scope.offline = false;
        $scope.hideFooter = false;
        $scope.rssEnabled = RSS.enabled;

        $scope.isActive = function (route) {
            return route === $location.path();
        };

        $scope.$on('offline', function (event, state) {
            $scope.$apply(function () {
                $scope.offline = state;
            });
        });

        $scope.$on('keyboard.show', function (event) {
            $scope.hideFooter = true;
        });

        $scope.$on('keyboard.hide', function (event) {
            $scope.hideFooter = false;
        });
    }
]);

controllers.controller('SearchController', [
    '$rootScope', '$scope', '$timeout', 'Monitor', '$geolocation', 'settings', '_', 'tracking', 'GPS', 'BannerService', '$cordovaDialogs',
    function ($rootScope, $scope, $timeout, Monitor, $geolocation, settings, _, tracking, GPS, BannerService, $cordovaDialogs) {
        $scope.customSearch = false;
        $scope.dropdownMenu = false;
        $scope.search = {};
        $scope.result = {};
        $scope.sorten = Monitor.getSorten();
        $scope.loading = true;
        $scope.bannerTop = '';

        $scope.init = function () {
            settings.get(function (settings) {
                $timeout(function () {
                    $scope.search = angular.copy(settings.getSearch());
                    if (_.isUndefined($scope.search.firstrun)) {
                        $scope.showCustomSearch();
                    } else {
                        $scope.searchWithGPS();
                    }
                });
            });

            $timeout(function () {
                BannerService.getBanner().then(function (banner) {
                    $scope.bannerTop = banner;
                });
            });
        };

        $scope.refreshSearch = function () {
            $scope.toggleDropdownMenu(false);

            if ($scope.customSearch) {
                $scope.searchWithAddress();
            } else {
                $scope.searchWithGPS(true);
            }
        };

        /**
         * @param Object sorte
         * @returns void
         */
        $scope.updateSorte = function (sorte) {
            $scope.search.sorte = sorte;
            $scope.refreshSearch();
        };

        /**
         * @param string query
         * @returns void
         */
        $scope.searchWithAddress = function (query) {
            tracking.trackEvent('App', 'Suche', 'manuell', 1);

            $scope.loading = true;
            $scope.result = {};

            if (query) {
                $scope.search.query = query;
            }

            $scope.updateResult($scope.search);
        };

        /**
         * @param boolean forceStandort
         * @returns void
         */
        $scope.searchWithGPS = function (forceStandort) {
            $scope.loading = true;
            $scope.result = {};

            if (true === $scope.search.standort || true === forceStandort) {
                $geolocation.getCurrentPosition({
                    maximumAge: GPS.maximumAge,
                    timeout: GPS.timeout,
                    enableHighAccuracy: false
                }).then(function (position) {
                    tracking.trackEvent('App', 'Suche', 'GPS', 1);

                    var query = position.coords.latitude + ',' + position.coords.longitude;
                    $scope.customSearch = false;
                    $scope.search.query = query;
                    $scope.updateResult($scope.search);
                }, function (error) {
                    $scope.showCustomSearch();
                });
            } else {
                $scope.showCustomSearch();
            }
        };

        /**
         * @param Object search
         * @returns void
         */
        $scope.updateResult = function (search) {
            Monitor.getTankstellen({
                query: search.query,
                order: search.sortierung ? search.sortierung.value : '',
                sorte: search.sorte ? search.sorte.value : $scope.sorten[0].value,
                radius: search.umkreis ? search.umkreis.value : '',
                marke: search.anbieter ? search.anbieter.name : '',
                limit: 20
            }).then(function (data) {
                $scope.result = data;
                $scope.loading = false;
            }, function (error) {
                $scope.showCustomSearch();
            });
        };

        $scope.hasElements = function (array) {
            if (_.isArray(array)) {
                return _.size(array) > 0;
            }
            return false;
        };

        $scope.isSorteActive = function (sorte) {
            if (!_.isUndefined($scope.search.sorte)) {
                return $scope.search.sorte.value == sorte.value;
            }
            return false;
        };

        $scope.toggleDropdownMenu = function (toggle) {
            $rootScope.$broadcast('menu.toggle.right', false);
            $rootScope.$broadcast('menu.toggle.dropdown', toggle);
        };

        $scope.showCustomSearch = function () {
            $scope.toggleDropdownMenu(false);
            $scope.customSearch = true;
            $scope.result = {};
            $scope.loading = false;
        };

        $scope.hideFooter = function (toggle) {
            $rootScope.$broadcast('app.toggle.footer', toggle);
        };

        $scope.$on('banner.top.changed', function () {
            BannerService.getBanner().then(function (banner) {
                $scope.bannerTop = banner;
            });
        });

        $scope.$on('settings.updated', function (event, settings) {
            $scope.search = angular.copy(settings.getSearch());
            $scope.refreshSearch();
        });

        $scope.$on('menu.toggle.dropdown', function (event, toggle) {
            if (_.isUndefined(toggle)) {
                $scope.dropdownMenu = !$scope.dropdownMenu;
            } else {
                $scope.dropdownMenu = toggle;
            }
        });

        $scope.init();
    }
]);

controllers.controller('RssController', [
    '$scope', '$window', '$http', 'RSS',
    function ($scope, $window, $http, RSS) {
        $scope.openLink = function (link) {
            $window.open(link, '_system', 'location=yes');
        };

        $scope.fetch = function () {
            $scope.loading = true;
            $http({method: 'GET', url: RSS.proxyurl, params: {feed: RSS.feed}, cache: true})
                .success(function (data, status) {
                    $scope.data = angular.fromJson(data);
                    $scope.loading = false;
                });
        };

        $scope.parseDate = function (dateString) {
            return Date.parse(dateString);
        };
    }
]);

controllers.controller('SettingsCtrl', [
    '$rootScope', '$scope', '$timeout', '$location', 'Monitor', '_', 'settings', 'log',
    function ($rootScope, $scope, $timeout, $location, Monitor, _, settings, log) {
        $scope.menuOpened = false;
        $scope.form = {};
        $scope.sorten = Monitor.getSorten();
        $scope.umkreise = Monitor.getUmkreise();
        $scope.sortierungen = Monitor.getSortierungen();

        $scope.anbieter = Monitor.getMarken().query(function () {
            settings.get(function (settings) {
                $timeout(function () {
                    var search = settings.getSearch();
                    $scope.form.anbieter = _.findWhere($scope.anbieter, search.anbieter);
                });
            });
        });

        settings.get(function (settings) {
            $timeout(function () {
                var search = settings.getSearch();
                $scope.form = {
                    standort: search.standort,
                    sortierung: _.findWhere($scope.sortierungen, search.sortierung),
                    umkreis: _.findWhere($scope.umkreise, search.umkreis),
                    sorte: _.findWhere($scope.sorten, search.sorte)
                };
            });
        });

        $scope.save = function (search) {
            search.firstrun = false;
            settings.update(search, function (settings) {
                $timeout(function () {
                    $rootScope.$broadcast('settings.updated', settings);
                    $location.path('/search');
                });
            }, function (e) {
                log.error(e);
            });
        };

        $scope.close = function () {
            $timeout(function () {
                $location.path('/search');
            });
        };

        $scope.$on('menu.toggle.right', function (event, toggle) {
            if (_.isUndefined(toggle)) {
                $scope.menuOpened = !$scope.menuOpened;
            } else {
                $scope.menuOpened = toggle;
            }
        });
    }
]);

controllers.controller('DebugController', [
    '$scope', 'log',
    function ($scope, log) {
        $scope.logs = log.get();
    }
]);