
angular.module('ngCordova', [
    'ngCordova.plugins'
]);


angular.module('ngCordova.plugins', [
    'ngCordova.plugins.device',
    'ngCordova.plugins.dialogs',
    'ngCordova.plugins.file',
    'ngCordova.plugins.googleAnalytics',
    'ngCordova.plugins.geolocation',
    'ngCordova.plugins.network',
    'ngCordova.plugins.splashscreen'
]);

//#### Begin Individual Plugin Code ####

// install   :      cordova plugin add cordova-plugin-network-information
// link      :      https://github.com/apache/cordova-plugin-network-information

/* globals Connection: true */
angular.module('ngCordova.plugins.network', [])

    .factory('$cordovaNetwork', ['$rootScope', '$timeout', function ($rootScope, $timeout) {

            /**
             * Fires offline a event
             */
            var offlineEvent = function () {
                var networkState = navigator.connection.type;
                $timeout(function () {
                    $rootScope.$broadcast('$cordovaNetwork:offline', networkState);
                });
            };

            /**
             * Fires online a event
             */
            var onlineEvent = function () {
                var networkState = navigator.connection.type;
                $timeout(function () {
                    $rootScope.$broadcast('$cordovaNetwork:online', networkState);
                });
            };

            document.addEventListener('deviceready', function () {
                if (navigator.connection) {
                    document.addEventListener('offline', offlineEvent, false);
                    document.addEventListener('online', onlineEvent, false);
                }
            });

            return {
                getNetwork: function () {
                    return navigator.connection.type;
                },
                isOnline: function () {
                    var networkState = navigator.connection.type;
                    return networkState !== Connection.UNKNOWN && networkState !== Connection.NONE;
                },
                isOffline: function () {
                    var networkState = navigator.connection.type;
                    return networkState === Connection.UNKNOWN || networkState === Connection.NONE;
                },
                clearOfflineWatch: function () {
                    document.removeEventListener('offline', offlineEvent);
                    $rootScope.$$listeners['$cordovaNetwork:offline'] = [];
                },
                clearOnlineWatch: function () {
                    document.removeEventListener('online', onlineEvent);
                    $rootScope.$$listeners['$cordovaNetwork:online'] = [];
                }
            };
        }]);
//    .run(['$injector', function ($injector) {
//            $injector.get('$cordovaNetwork'); //ensure the factory always gets initialised
//        }]);
// install   :      cordova plugin add cordova-plugin-splashscreen
// link      :      https://github.com/apache/cordova-plugin-splashscreen

angular.module('ngCordova.plugins.splashscreen', [])

    .factory('$cordovaSplashscreen', [function () {

            return {
                hide: function () {
                    return navigator.splashscreen.hide();
                },
                show: function () {
                    return navigator.splashscreen.show();
                }
            };

        }]);
// install   :     cordova plugin add cordova-plugin-device
// link      :     https://github.com/apache/cordova-plugin-device

/* globals device: true */
angular.module('ngCordova.plugins.device', [])

    .factory('$cordovaDevice', [function () {

            return {
                /**
                 * Returns the whole device object.
                 * @see https://github.com/apache/cordova-plugin-device
                 * @returns {Object} The device object.
                 */
                getDevice: function () {
                    return device;
                },
                /**
                 * Returns the Cordova version.
                 * @see https://github.com/apache/cordova-plugin-device#devicecordova
                 * @returns {String} The Cordova version.
                 */
                getCordova: function () {
                    return device.cordova;
                },
                /**
                 * Returns the name of the device's model or product.
                 * @see https://github.com/apache/cordova-plugin-device#devicemodel
                 * @returns {String} The name of the device's model or product.
                 */
                getModel: function () {
                    return device.model;
                },
                /**
                 * @deprecated device.name is deprecated as of version 2.3.0. Use device.model instead.
                 * @returns {String}
                 */
                getName: function () {
                    return device.name;
                },
                /**
                 * Returns the device's operating system name.
                 * @see https://github.com/apache/cordova-plugin-device#deviceplatform
                 * @returns {String} The device's operating system name.
                 */
                getPlatform: function () {
                    return device.platform;
                },
                /**
                 * Returns the device's Universally Unique Identifier.
                 * @see https://github.com/apache/cordova-plugin-device#deviceuuid
                 * @returns {String} The device's Universally Unique Identifier
                 */
                getUUID: function () {
                    return device.uuid;
                },
                /**
                 * Returns the operating system version.
                 * @see https://github.com/apache/cordova-plugin-device#deviceversion
                 * @returns {String}
                 */
                getVersion: function () {
                    return device.version;
                },
                /**
                 * Returns the device manufacturer.
                 * @returns {String}
                 */
                getManufacturer: function () {
                    return device.manufacturer;
                }
            };
        }]);
// install   :     cordova plugin add cordova-plugin-dialogs
// link      :     https://github.com/apache/cordova-plugin-dialogs

angular.module('ngCordova.plugins.dialogs', [])

    .factory('$cordovaDialogs', ['$q', '$window', function ($q, $window) {

            return {
                alert: function (message, title, buttonName) {
                    var q = $q.defer();

                    if (!$window.navigator.notification) {
                        $window.alert(message);
                        q.resolve();
                    } else {
                        navigator.notification.alert(message, function () {
                            q.resolve();
                        }, title, buttonName);
                    }

                    return q.promise;
                },
                confirm: function (message, title, buttonLabels) {
                    var q = $q.defer();

                    if (!$window.navigator.notification) {
                        if ($window.confirm(message)) {
                            q.resolve(1);
                        } else {
                            q.resolve(2);
                        }
                    } else {
                        navigator.notification.confirm(message, function (buttonIndex) {
                            q.resolve(buttonIndex);
                        }, title, buttonLabels);
                    }

                    return q.promise;
                },
                prompt: function (message, title, buttonLabels, defaultText) {
                    var q = $q.defer();

                    if (!$window.navigator.notification) {
                        var res = $window.prompt(message, defaultText);
                        if (res !== null) {
                            q.resolve({input1: res, buttonIndex: 1});
                        } else {
                            q.resolve({input1: res, buttonIndex: 2});
                        }
                    } else {
                        navigator.notification.prompt(message, function (result) {
                            q.resolve(result);
                        }, title, buttonLabels, defaultText);
                    }
                    return q.promise;
                },
                beep: function (times) {
                    return navigator.notification.beep(times);
                },
                activityStart: function (message, title) {
                    var q = $q.defer();

                    if (cordova.platformId === 'android') {
                        navigator.notification.activityStart(title, message);
                        q.resolve();
                    } else {
                        q.reject(message, title);
                    }

                    return q.promise;
                },
                activityStop: function () {
                    var q = $q.defer();

                    if (cordova.platformId === 'android') {
                        navigator.notification.activityStop();
                        q.resolve();
                    } else {
                        q.reject();
                    }

                    return q.promise;
                },
                progressStart: function (message, title) {
                    var q = $q.defer();

                    if (cordova.platformId === 'android') {
                        navigator.notification.progressStart(title, message);
                        q.resolve();
                    } else {
                        q.reject(message, title);
                    }

                    return q.promise;
                },
                progressStop: function () {
                    var q = $q.defer();

                    if (cordova.platformId === 'android') {
                        navigator.notification.progressStop();
                        q.resolve();
                    } else {
                        q.reject();
                    }

                    return q.promise;
                },
                progressValue: function (value) {
                    var q = $q.defer();

                    if (cordova.platformId === 'android') {
                        navigator.notification.progressValue(value);
                        q.resolve();
                    } else {
                        q.reject(value);
                    }

                    return q.promise;
                }
            };
        }]);
// install   :     cordova plugin add cordova-plugin-file
// link      :     https://github.com/apache/cordova-plugin-file

angular.module('ngCordova.plugins.file', [])

    .constant('$cordovaFileError', {
        1: 'NOT_FOUND_ERR',
        2: 'SECURITY_ERR',
        3: 'ABORT_ERR',
        4: 'NOT_READABLE_ERR',
        5: 'ENCODING_ERR',
        6: 'NO_MODIFICATION_ALLOWED_ERR',
        7: 'INVALID_STATE_ERR',
        8: 'SYNTAX_ERR',
        9: 'INVALID_MODIFICATION_ERR',
        10: 'QUOTA_EXCEEDED_ERR',
        11: 'TYPE_MISMATCH_ERR',
        12: 'PATH_EXISTS_ERR'
    })

    .provider('$cordovaFile', [function () {

            this.$get = ['$q', '$window', '$cordovaFileError', function ($q, $window, $cordovaFileError) {

                    return {
                        getFreeDiskSpace: function () {
                            var q = $q.defer();
                            cordova.exec(function (result) {
                                q.resolve(result);
                            }, function (error) {
                                q.reject(error);
                            }, 'File', 'getFreeDiskSpace', []);
                            return q.promise;
                        },
                        checkDir: function (path, dir) {
                            var q = $q.defer();

                            if ((/^\//.test(dir))) {
                                q.reject('directory cannot start with \/');
                            }

                            try {
                                var directory = path + dir;
                                $window.resolveLocalFileSystemURL(directory, function (fileSystem) {
                                    if (fileSystem.isDirectory === true) {
                                        q.resolve(fileSystem);
                                    } else {
                                        q.reject({code: 13, message: 'input is not a directory'});
                                    }
                                }, function (error) {
                                    error.message = $cordovaFileError[error.code];
                                    q.reject(error);
                                });
                            } catch (err) {
                                err.message = $cordovaFileError[err.code];
                                q.reject(err);
                            }

                            return q.promise;
                        },
                        checkFile: function (path, file) {
                            var q = $q.defer();

                            if ((/^\//.test(file))) {
                                q.reject('directory cannot start with \/');
                            }

                            try {
                                var directory = path + file;
                                $window.resolveLocalFileSystemURL(directory, function (fileSystem) {
                                    if (fileSystem.isFile === true) {
                                        q.resolve(fileSystem);
                                    } else {
                                        q.reject({code: 13, message: 'input is not a file'});
                                    }
                                }, function (error) {
                                    error.message = $cordovaFileError[error.code];
                                    q.reject(error);
                                });
                            } catch (err) {
                                err.message = $cordovaFileError[err.code];
                                q.reject(err);
                            }

                            return q.promise;
                        },
                        createDir: function (path, dirName, replaceBool) {
                            var q = $q.defer();

                            if ((/^\//.test(dirName))) {
                                q.reject('directory cannot start with \/');
                            }

                            replaceBool = replaceBool ? false : true;

                            var options = {
                                create: true,
                                exclusive: replaceBool
                            };

                            try {
                                $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                                    fileSystem.getDirectory(dirName, options, function (result) {
                                        q.resolve(result);
                                    }, function (error) {
                                        error.message = $cordovaFileError[error.code];
                                        q.reject(error);
                                    });
                                }, function (err) {
                                    err.message = $cordovaFileError[err.code];
                                    q.reject(err);
                                });
                            } catch (e) {
                                e.message = $cordovaFileError[e.code];
                                q.reject(e);
                            }

                            return q.promise;
                        },
                        createFile: function (path, fileName, replaceBool) {
                            var q = $q.defer();

                            if ((/^\//.test(fileName))) {
                                q.reject('file-name cannot start with \/');
                            }

                            replaceBool = replaceBool ? false : true;

                            var options = {
                                create: true,
                                exclusive: replaceBool
                            };

                            try {
                                $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                                    fileSystem.getFile(fileName, options, function (result) {
                                        q.resolve(result);
                                    }, function (error) {
                                        error.message = $cordovaFileError[error.code];
                                        q.reject(error);
                                    });
                                }, function (err) {
                                    err.message = $cordovaFileError[err.code];
                                    q.reject(err);
                                });
                            } catch (e) {
                                e.message = $cordovaFileError[e.code];
                                q.reject(e);
                            }
                            return q.promise;
                        },
                        removeDir: function (path, dirName) {
                            var q = $q.defer();

                            if ((/^\//.test(dirName))) {
                                q.reject('file-name cannot start with \/');
                            }

                            try {
                                $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                                    fileSystem.getDirectory(dirName, {create: false}, function (dirEntry) {
                                        dirEntry.remove(function () {
                                            q.resolve({success: true, fileRemoved: dirEntry});
                                        }, function (error) {
                                            error.message = $cordovaFileError[error.code];
                                            q.reject(error);
                                        });
                                    }, function (err) {
                                        err.message = $cordovaFileError[err.code];
                                        q.reject(err);
                                    });
                                }, function (er) {
                                    er.message = $cordovaFileError[er.code];
                                    q.reject(er);
                                });
                            } catch (e) {
                                e.message = $cordovaFileError[e.code];
                                q.reject(e);
                            }
                            return q.promise;
                        },
                        removeFile: function (path, fileName) {
                            var q = $q.defer();

                            if ((/^\//.test(fileName))) {
                                q.reject('file-name cannot start with \/');
                            }

                            try {
                                $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                                    fileSystem.getFile(fileName, {create: false}, function (fileEntry) {
                                        fileEntry.remove(function () {
                                            q.resolve({success: true, fileRemoved: fileEntry});
                                        }, function (error) {
                                            error.message = $cordovaFileError[error.code];
                                            q.reject(error);
                                        });
                                    }, function (err) {
                                        err.message = $cordovaFileError[err.code];
                                        q.reject(err);
                                    });
                                }, function (er) {
                                    er.message = $cordovaFileError[er.code];
                                    q.reject(er);
                                });
                            } catch (e) {
                                e.message = $cordovaFileError[e.code];
                                q.reject(e);
                            }
                            return q.promise;
                        },
                        removeRecursively: function (path, dirName) {
                            var q = $q.defer();

                            if ((/^\//.test(dirName))) {
                                q.reject('file-name cannot start with \/');
                            }

                            try {
                                $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                                    fileSystem.getDirectory(dirName, {create: false}, function (dirEntry) {
                                        dirEntry.removeRecursively(function () {
                                            q.resolve({success: true, fileRemoved: dirEntry});
                                        }, function (error) {
                                            error.message = $cordovaFileError[error.code];
                                            q.reject(error);
                                        });
                                    }, function (err) {
                                        err.message = $cordovaFileError[err.code];
                                        q.reject(err);
                                    });
                                }, function (er) {
                                    er.message = $cordovaFileError[er.code];
                                    q.reject(er);
                                });
                            } catch (e) {
                                e.message = $cordovaFileError[e.code];
                                q.reject(e);
                            }
                            return q.promise;
                        },
                        writeFile: function (path, fileName, text, replaceBool) {
                            var q = $q.defer();

                            if ((/^\//.test(fileName))) {
                                q.reject('file-name cannot start with \/');
                            }

                            replaceBool = replaceBool ? false : true;

                            var options = {
                                create: true,
                                exclusive: replaceBool
                            };

                            try {
                                $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                                    fileSystem.getFile(fileName, options, function (fileEntry) {
                                        fileEntry.createWriter(function (writer) {
                                            if (options.append === true) {
                                                writer.seek(writer.length);
                                            }

                                            if (options.truncate) {
                                                writer.truncate(options.truncate);
                                            }

                                            writer.onwriteend = function (evt) {
                                                if (this.error) {
                                                    q.reject(this.error);
                                                } else {
                                                    q.resolve(evt);
                                                }
                                            };

                                            writer.write(text);

                                            q.promise.abort = function () {
                                                writer.abort();
                                            };
                                        });
                                    }, function (error) {
                                        error.message = $cordovaFileError[error.code];
                                        q.reject(error);
                                    });
                                }, function (err) {
                                    err.message = $cordovaFileError[err.code];
                                    q.reject(err);
                                });
                            } catch (e) {
                                e.message = $cordovaFileError[e.code];
                                q.reject(e);
                            }

                            return q.promise;
                        },
                        writeExistingFile: function (path, fileName, text) {
                            var q = $q.defer();

                            if ((/^\//.test(fileName))) {
                                q.reject('file-name cannot start with \/');
                            }

                            try {
                                $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                                    fileSystem.getFile(fileName, {create: false}, function (fileEntry) {
                                        fileEntry.createWriter(function (writer) {
                                            writer.seek(writer.length);

                                            writer.onwriteend = function (evt) {
                                                if (this.error) {
                                                    q.reject(this.error);
                                                } else {
                                                    q.resolve(evt);
                                                }
                                            };

                                            writer.write(text);

                                            q.promise.abort = function () {
                                                writer.abort();
                                            };
                                        });
                                    }, function (error) {
                                        error.message = $cordovaFileError[error.code];
                                        q.reject(error);
                                    });
                                }, function (err) {
                                    err.message = $cordovaFileError[err.code];
                                    q.reject(err);
                                });
                            } catch (e) {
                                e.message = $cordovaFileError[e.code];
                                q.reject(e);
                            }

                            return q.promise;
                        },
                        readAsText: function (path, file) {
                            var q = $q.defer();

                            if ((/^\//.test(file))) {
                                q.reject('file-name cannot start with \/');
                            }

                            try {
                                $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                                    fileSystem.getFile(file, {create: false}, function (fileEntry) {
                                        fileEntry.file(function (fileData) {
                                            var reader = new FileReader();

                                            reader.onloadend = function (evt) {
                                                if (evt.target.result !== undefined || evt.target.result !== null) {
                                                    q.resolve(evt.target.result);
                                                } else if (evt.target.error !== undefined || evt.target.error !== null) {
                                                    q.reject(evt.target.error);
                                                } else {
                                                    q.reject({code: null, message: 'READER_ONLOADEND_ERR'});
                                                }
                                            };

                                            reader.readAsText(fileData);
                                        });
                                    }, function (error) {
                                        error.message = $cordovaFileError[error.code];
                                        q.reject(error);
                                    });
                                }, function (err) {
                                    err.message = $cordovaFileError[err.code];
                                    q.reject(err);
                                });
                            } catch (e) {
                                e.message = $cordovaFileError[e.code];
                                q.reject(e);
                            }

                            return q.promise;
                        },
                        readAsDataURL: function (path, file) {
                            var q = $q.defer();

                            if ((/^\//.test(file))) {
                                q.reject('file-name cannot start with \/');
                            }

                            try {
                                $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                                    fileSystem.getFile(file, {create: false}, function (fileEntry) {
                                        fileEntry.file(function (fileData) {
                                            var reader = new FileReader();
                                            reader.onloadend = function (evt) {
                                                if (evt.target.result !== undefined || evt.target.result !== null) {
                                                    q.resolve(evt.target.result);
                                                } else if (evt.target.error !== undefined || evt.target.error !== null) {
                                                    q.reject(evt.target.error);
                                                } else {
                                                    q.reject({code: null, message: 'READER_ONLOADEND_ERR'});
                                                }
                                            };
                                            reader.readAsDataURL(fileData);
                                        });
                                    }, function (error) {
                                        error.message = $cordovaFileError[error.code];
                                        q.reject(error);
                                    });
                                }, function (err) {
                                    err.message = $cordovaFileError[err.code];
                                    q.reject(err);
                                });
                            } catch (e) {
                                e.message = $cordovaFileError[e.code];
                                q.reject(e);
                            }

                            return q.promise;
                        },
                        readAsBinaryString: function (path, file) {
                            var q = $q.defer();

                            if ((/^\//.test(file))) {
                                q.reject('file-name cannot start with \/');
                            }

                            try {
                                $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                                    fileSystem.getFile(file, {create: false}, function (fileEntry) {
                                        fileEntry.file(function (fileData) {
                                            var reader = new FileReader();
                                            reader.onloadend = function (evt) {
                                                if (evt.target.result !== undefined || evt.target.result !== null) {
                                                    q.resolve(evt.target.result);
                                                } else if (evt.target.error !== undefined || evt.target.error !== null) {
                                                    q.reject(evt.target.error);
                                                } else {
                                                    q.reject({code: null, message: 'READER_ONLOADEND_ERR'});
                                                }
                                            };
                                            reader.readAsBinaryString(fileData);
                                        });
                                    }, function (error) {
                                        error.message = $cordovaFileError[error.code];
                                        q.reject(error);
                                    });
                                }, function (err) {
                                    err.message = $cordovaFileError[err.code];
                                    q.reject(err);
                                });
                            } catch (e) {
                                e.message = $cordovaFileError[e.code];
                                q.reject(e);
                            }

                            return q.promise;
                        },
                        readAsArrayBuffer: function (path, file) {
                            var q = $q.defer();

                            if ((/^\//.test(file))) {
                                q.reject('file-name cannot start with \/');
                            }

                            try {
                                $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                                    fileSystem.getFile(file, {create: false}, function (fileEntry) {
                                        fileEntry.file(function (fileData) {
                                            var reader = new FileReader();
                                            reader.onloadend = function (evt) {
                                                if (evt.target.result !== undefined || evt.target.result !== null) {
                                                    q.resolve(evt.target.result);
                                                } else if (evt.target.error !== undefined || evt.target.error !== null) {
                                                    q.reject(evt.target.error);
                                                } else {
                                                    q.reject({code: null, message: 'READER_ONLOADEND_ERR'});
                                                }
                                            };
                                            reader.readAsArrayBuffer(fileData);
                                        });
                                    }, function (error) {
                                        error.message = $cordovaFileError[error.code];
                                        q.reject(error);
                                    });
                                }, function (err) {
                                    err.message = $cordovaFileError[err.code];
                                    q.reject(err);
                                });
                            } catch (e) {
                                e.message = $cordovaFileError[e.code];
                                q.reject(e);
                            }

                            return q.promise;
                        },
                        moveFile: function (path, fileName, newPath, newFileName) {
                            var q = $q.defer();

                            newFileName = newFileName || fileName;

                            if ((/^\//.test(fileName)) || (/^\//.test(newFileName))) {
                                q.reject('file-name cannot start with \/');
                            }

                            try {
                                $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                                    fileSystem.getFile(fileName, {create: false}, function (fileEntry) {
                                        $window.resolveLocalFileSystemURL(newPath, function (newFileEntry) {
                                            fileEntry.moveTo(newFileEntry, newFileName, function (result) {
                                                q.resolve(result);
                                            }, function (error) {
                                                q.reject(error);
                                            });
                                        }, function (err) {
                                            q.reject(err);
                                        });
                                    }, function (err) {
                                        q.reject(err);
                                    });
                                }, function (er) {
                                    q.reject(er);
                                });
                            } catch (e) {
                                q.reject(e);
                            }
                            return q.promise;
                        },
                        moveDir: function (path, dirName, newPath, newDirName) {
                            var q = $q.defer();

                            newDirName = newDirName || dirName;

                            if (/^\//.test(dirName) || (/^\//.test(newDirName))) {
                                q.reject('file-name cannot start with \/');
                            }

                            try {
                                $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                                    fileSystem.getDirectory(dirName, {create: false}, function (dirEntry) {
                                        $window.resolveLocalFileSystemURL(newPath, function (newDirEntry) {
                                            dirEntry.moveTo(newDirEntry, newDirName, function (result) {
                                                q.resolve(result);
                                            }, function (error) {
                                                q.reject(error);
                                            });
                                        }, function (erro) {
                                            q.reject(erro);
                                        });
                                    }, function (err) {
                                        q.reject(err);
                                    });
                                }, function (er) {
                                    q.reject(er);
                                });
                            } catch (e) {
                                q.reject(e);
                            }
                            return q.promise;
                        },
                        copyDir: function (path, dirName, newPath, newDirName) {
                            var q = $q.defer();

                            newDirName = newDirName || dirName;

                            if (/^\//.test(dirName) || (/^\//.test(newDirName))) {
                                q.reject('file-name cannot start with \/');
                            }

                            try {
                                $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                                    fileSystem.getDirectory(dirName, {create: false, exclusive: false}, function (dirEntry) {

                                        $window.resolveLocalFileSystemURL(newPath, function (newDirEntry) {
                                            dirEntry.copyTo(newDirEntry, newDirName, function (result) {
                                                q.resolve(result);
                                            }, function (error) {
                                                error.message = $cordovaFileError[error.code];
                                                q.reject(error);
                                            });
                                        }, function (erro) {
                                            erro.message = $cordovaFileError[erro.code];
                                            q.reject(erro);
                                        });
                                    }, function (err) {
                                        err.message = $cordovaFileError[err.code];
                                        q.reject(err);
                                    });
                                }, function (er) {
                                    er.message = $cordovaFileError[er.code];
                                    q.reject(er);
                                });
                            } catch (e) {
                                e.message = $cordovaFileError[e.code];
                                q.reject(e);
                            }
                            return q.promise;
                        },
                        copyFile: function (path, fileName, newPath, newFileName) {
                            var q = $q.defer();

                            newFileName = newFileName || fileName;

                            if ((/^\//.test(fileName))) {
                                q.reject('file-name cannot start with \/');
                            }

                            try {
                                $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                                    fileSystem.getFile(fileName, {create: false, exclusive: false}, function (fileEntry) {

                                        $window.resolveLocalFileSystemURL(newPath, function (newFileEntry) {
                                            fileEntry.copyTo(newFileEntry, newFileName, function (result) {
                                                q.resolve(result);
                                            }, function (error) {
                                                error.message = $cordovaFileError[error.code];
                                                q.reject(error);
                                            });
                                        }, function (erro) {
                                            erro.message = $cordovaFileError[erro.code];
                                            q.reject(erro);
                                        });
                                    }, function (err) {
                                        err.message = $cordovaFileError[err.code];
                                        q.reject(err);
                                    });
                                }, function (er) {
                                    er.message = $cordovaFileError[er.code];
                                    q.reject(er);
                                });
                            } catch (e) {
                                e.message = $cordovaFileError[e.code];
                                q.reject(e);
                            }
                            return q.promise;
                        },
                        readFileMetadata: function (path, file) {
                            var q = $q.defer();

                            if ((/^\//.test(file))) {
                                q.reject('directory cannot start with \/');
                            }

                            try {
                                var directory = path + file;
                                $window.resolveLocalFileSystemURL(directory, function (fileEntry) {
                                    fileEntry.file(function (result) {
                                        q.resolve(result);
                                    }, function (error) {
                                        error.message = $cordovaFileError[error.code];
                                        q.reject(error);
                                    });
                                }, function (err) {
                                    err.message = $cordovaFileError[err.code];
                                    q.reject(err);
                                });
                            } catch (e) {
                                e.message = $cordovaFileError[e.code];
                                q.reject(e);
                            }

                            return q.promise;
                        }

                        /*
                         listFiles: function (path, dir) {
                         
                         },
                         
                         listDir: function (path, dirName) {
                         var q = $q.defer();
                         
                         try {
                         $window.resolveLocalFileSystemURL(path, function (fileSystem) {
                         fileSystem.getDirectory(dirName, options, function (parent) {
                         var reader = parent.createReader();
                         reader.readEntries(function (entries) {
                         q.resolve(entries);
                         }, function () {
                         q.reject('DIR_READ_ERROR : ' + path + dirName);
                         });
                         }, function (error) {
                         error.message = $cordovaFileError[error.code];
                         q.reject(error);
                         });
                         }, function (err) {
                         err.message = $cordovaFileError[err.code];
                         q.reject(err);
                         });
                         } catch (e) {
                         e.message = $cordovaFileError[e.code];
                         q.reject(e);
                         }
                         
                         return q.promise;
                         },
                         
                         */
                    };

                }];
        }]);
// install   :     cordova plugin add cordova-plugin-geolocation
// link      :     https://github.com/apache/cordova-plugin-geolocation

angular.module('ngCordova.plugins.geolocation', [])

    .factory('$cordovaGeolocation', ['$q', function ($q) {

            return {
                getCurrentPosition: function (options) {
                    var q = $q.defer();

                    navigator.geolocation.getCurrentPosition(function (result) {
                        q.resolve(result);
                    }, function (err) {
                        q.reject(err);
                    }, options);

                    return q.promise;
                },
                watchPosition: function (options) {
                    var q = $q.defer();

                    var watchID = navigator.geolocation.watchPosition(function (result) {
                        q.notify(result);
                    }, function (err) {
                        q.reject(err);
                    }, options);

                    q.promise.cancel = function () {
                        navigator.geolocation.clearWatch(watchID);
                    };

                    q.promise.clearWatch = function (id) {
                        navigator.geolocation.clearWatch(id || watchID);
                    };

                    q.promise.watchID = watchID;

                    return q.promise;
                },
                clearWatch: function (watchID) {
                    return navigator.geolocation.clearWatch(watchID);
                }
            };
        }]);
// install   :     cordova plugin add https://github.com/danwilson/google-analytics-plugin.git
// link      :     https://github.com/danwilson/google-analytics-plugin

angular.module('ngCordova.plugins.googleAnalytics', [])

    .factory('$cordovaGoogleAnalytics', ['$q', '$window', function ($q, $window) {

            return {
                startTrackerWithId: function (id) {
                    var d = $q.defer();

                    $window.analytics.startTrackerWithId(id, function (response) {
                        d.resolve(response);
                    }, function (error) {
                        d.reject(error);
                    });

                    return d.promise;
                },
                setUserId: function (id) {
                    var d = $q.defer();

                    $window.analytics.setUserId(id, function (response) {
                        d.resolve(response);
                    }, function (error) {
                        d.reject(error);
                    });

                    return d.promise;
                },
                debugMode: function () {
                    var d = $q.defer();

                    $window.analytics.debugMode(function (response) {
                        d.resolve(response);
                    }, function () {
                        d.reject();
                    });

                    return d.promise;
                },
                trackView: function (screenName) {
                    var d = $q.defer();

                    $window.analytics.trackView(screenName, function (response) {
                        d.resolve(response);
                    }, function (error) {
                        d.reject(error);
                    });

                    return d.promise;
                },
                addCustomDimension: function (key, value) {
                    var d = $q.defer();
                    var parsedKey = parseInt(key, 10);

                    if (isNaN(parsedKey)) {
                        d.reject('Parameter "key" must be an integer.');
                    }

                    $window.analytics.addCustomDimension(parsedKey, value, function () {
                        d.resolve();
                    }, function (error) {
                        d.reject(error);
                    });

                    return d.promise;
                },
                trackEvent: function (category, action, label, value) {
                    var d = $q.defer();

                    $window.analytics.trackEvent(category, action, label, value, function (response) {
                        d.resolve(response);
                    }, function (error) {
                        d.reject(error);
                    });

                    return d.promise;
                },
                trackException: function (description, fatal) {
                    var d = $q.defer();

                    $window.analytics.trackException(description, fatal, function (response) {
                        d.resolve(response);
                    }, function (error) {
                        d.reject(error);
                    });

                    return d.promise;
                },
                trackTiming: function (category, milliseconds, variable, label) {
                    var d = $q.defer();

                    $window.analytics.trackTiming(category, milliseconds, variable, label, function (response) {
                        d.resolve(response);
                    }, function (error) {
                        d.reject(error);
                    });

                    return d.promise;
                },
                addTransaction: function (transactionId, affiliation, revenue, tax, shipping, currencyCode) {
                    var d = $q.defer();

                    $window.analytics.addTransaction(transactionId, affiliation, revenue, tax, shipping, currencyCode, function (response) {
                        d.resolve(response);
                    }, function (error) {
                        d.reject(error);
                    });

                    return d.promise;
                },
                addTransactionItem: function (transactionId, name, sku, category, price, quantity, currencyCode) {
                    var d = $q.defer();

                    $window.analytics.addTransactionItem(transactionId, name, sku, category, price, quantity, currencyCode, function (response) {
                        d.resolve(response);
                    }, function (error) {
                        d.reject(error);
                    });

                    return d.promise;
                }
            };
        }]);
