'use strict';

angular.module('phonegap', [])

    .directive('externallink', [
        '$window',
        function ($window) {
            return {
                restrict: 'AC',
                link: function (scope, elem, attrs) {
                    elem.bind('click', function (event) {
                        $window.open(elem.attr('href'), '_system', 'location=yes');
                        return event.preventDefault();
                    });
                }
            };
        }
    ])

    .factory('$geolocation', function ($q, $platform) {
        return {
            getCurrentPosition: function (options) {
                var d = $q.defer();

                $platform.ready(function () {
                    navigator.geolocation.getCurrentPosition(function (result) {
                        d.resolve(result);
                    }, function (err) {
                        d.reject(err);
                    }, options);
                });

                return d.promise;
            }
        };
    })

    /** DETECTING ON/OFFLINE STATUS **/
    .run([
        '$rootScope', '$document', '$platform', '$cordovaNetwork', '$timeout',
        function ($rootScope, $document, $platform, $cordovaNetwork, $timeout) {
            $platform.ready(function () {
                if (!$platform.is('browser')) {
                    $timeout(function () {
                        $rootScope.$broadcast('offline', $cordovaNetwork.isOffline());
                    });
                }
            });
            $document.on('offline', function () {
                $rootScope.$broadcast('offline', true);
            });

            $document.on('online', function () {
                $rootScope.$broadcast('offline', false);
            });
        }
    ])

    /** HIDE SPLASHSCREEN ON DOM IS READY **/
    .run([
        '$timeout',
        function ($timeout) {
            $timeout(function () {
                if (navigator.splashscreen) {
                    navigator.splashscreen.hide();
                }
            });
        }
    ])

    /** Listing to Android's menu (button) event **/
    .run([
        '$rootScope', '$document', '$timeout',
        function ($rootScope, $document, $timeout) {
            $document.on('deviceready', function () {
                $document.on('menubutton', function () {
                    $timeout(function () {
                        $rootScope.$broadcast('menu.toggle.dropdown');
                    });
                });
            });
        }
    ])

    /** Listing hide/show keyboard event **/
    .run([
        '$rootScope', '$timeout',
        function ($rootScope, $timeout) {
            window.addEventListener('native.keyboardhide', function () {
                $timeout(function () {
                    $rootScope.$broadcast('keyboard.hide');
                });
            });
            window.addEventListener('native.keyboardshow', function () {
                $timeout(function () {
                    $rootScope.$broadcast('keyboard.show');
                });
            });
        }
    ]);
        