'use strict';

angular.module('spmApp.services')
    .service('Favorites', function($q, $http, FAVS, DatabaseService) {
        var that = this;
        this.getBookmarks = function(user) {
            var deffered = $q.defer();
            var params = {user: this.md5(user.user), token: user.token};

            $http.get(FAVS.bookmarksUri, {params: params})
                .success(function(data, status) {
                    deffered.resolve(data, status);
                })
                .error(function(data, status) {
                    if (status === 403) {
                        that.logout();
                    }
                    deffered.reject(data, status);
                });

            return deffered.promise;
        };
        this.addBookmark = function(user, id) {
            var deffered = $q.defer();
            var params = {mtskId: id, user: this.md5(user.user), token: user.token};
            var config = {headers: {'Content-Type': 'application/x-www-form-urlencoded'}, params: params};

            $http.post(FAVS.bookmarksUri, '', config)
                .success(function(data, status) {
                    deffered.resolve(data, status);
                })
                .error(function(data, status) {
                    if (status === 403) {
                        that.logout();
                    }
                    deffered.reject(data, status);
                });

            return deffered.promise;
        };
        this.deleteBookmark = function(user, id) {
            var deffered = $q.defer();
            var params = {mtskId: id, user: this.md5(user.user), token: user.token};
            var config = {headers: {'Content-Type': 'application/x-www-form-urlencoded'}, params: params};

            $http.post(FAVS.bookmarksUri, '_method=DELETE', config)
                .success(function(data, status) {
                    deffered.resolve(data, status);
                })
                .error(function(data, status) {
                    if (status === 403) {
                        that.logout();
                    }
                    deffered.reject(data, status);
                });

            return deffered.promise;
        };
        this.isAuthenticated = function() {
            var deffered = $q.defer();
            DatabaseService.get('kasuser')
                .then(function(user) {
                    deffered.resolve(user);
                }, function() {
                    deffered.reject();
                });

            return deffered.promise;
        };
        this.authenticate = function(user, password) {
            var deffered = $q.defer();
            var config = {headers: {'Content-Type': 'application/x-www-form-urlencoded'}};

            $http.post(FAVS.authUri, 'user=' + user + '&password=' + password, config)
                .success(function(data, status) {
                    user = {user: user, token: data.token};
                    DatabaseService.save('kasuser', user)
                        .then(function() {
                            deffered.resolve(user);
                        }, function() {
                            deffered.reject();
                        });
                })
                .error(function() {
                    deffered.reject();
                });

            return deffered.promise;
        };
        this.logout = function() {
            var deffered = $q.defer();
            DatabaseService.save('kasuser', '')
                .then(function() {
                    deffered.resolve();
                }, function() {
                    deffered.reject();
                });

            return deffered.promise;
        };
        this.md5 = function(value) {
            return CryptoJS.MD5(value).toString();
        };
        this.getPasswordUri = function() {
            return FAVS.passwordUri;
        };
        this.getRegisterUri = function() {
            return FAVS.registerUri;
        };
    });