'use strict';

angular.module('spmApp.services')
    .service('Monitor', function(_, $q, $http, $resource, API) {
        this.getTankstellen = function(params) {
            var deffered = $q.defer();
            $resource(API.uri + 'tankstellen', {apiKey: API.key}, {
                query: {method: 'GET', isArray: true}
            }).query(params, function(data) {
                var tankstellen = [];
                angular.forEach(angular.fromJson(data), function(item) {
                    // iOS workaround
                    var arr = item.datum.split(/[- :]/);
                    item.datum = new Date(arr[0], arr[1] - 1, arr[2], arr[3], arr[4], arr[5]);
                    tankstellen.push(item);
                });

                deffered.resolve(tankstellen);
            }, function(error) {
                deffered.reject(error);
            });

            return deffered.promise;
        };
        this.getTankstellenByIds = function(ids, sorte) {
            var deffered = $q.defer();

            this.getTankstellen({mtskIds: ids.join(','), sorte: sorte})
                .then(function(tankstellen) {
                    tankstellen = _.sortBy(tankstellen, function(tankstelle) {
                        return tankstelle[sorte];
                    });

                    deffered.resolve(tankstellen);
                }, function(error) {
                    deffered.reject(error);
                });

            return deffered.promise;
        };
        this.getMarken = function() {
            return $resource(API.uri + 'marken', {apiKey: API.key}, {
                query: {
                    method: 'GET',
                    isArray: true,
                    transformResponse: function(data, headersGetter) {
                        var marken = [];
                        angular.forEach(angular.fromJson(data), function(value) {
                            marken.push({name: value.toString()});
                        });
                        return marken;
                    }
                }
            });
        };
        this.getZeiten = function(tankstelle) {
            var deffered = $q.defer();
            $http({
                method: 'GET',
                url: API.uri + 'oeffnungszeiten',
                params: {apiKey: API.key, id: tankstelle.mtsk_id},
                cache: true
            }).success(function(data, status) {
                deffered.resolve(data);
            }).error(function(data, status) {
                deffered.reject(data);
            });

            return deffered.promise;
        };
        this.getSorten = function() {
            return [
                {name: "Diesel", value: 'diesel'},
                {name: "Super", value: 'e5'},
                {name: "Super (E10)", value: 'e10'}
            ];
        };
        this.getUmkreise = function() {
            return [
                {name: '2 km', value: 2},
                {name: '5 km', value: 5},
                {name: '10 km', value: 10},
                {name: '25 km', value: 25},
                {name: '50 km', value: 50}
            ];
        };
        this.getSortierungen = function() {
            return [
                {name: 'Sortierung nach Preis', value: 'preis'},
                {name: 'Sortierung nach km', value: 'entfernung'}
            ];
        };
    });