'use strict';

var services = angular.module('spmApp.services', ['ngResource', 'phonegap']);

services.value('version', '0.1');

services.factory('_', function() {
    return window._;
});

services.factory('base64', function() {
    return {
        encode: function(value) {
            return btoa(value);
        },
        decode: function(value) {
            return atob(value);
        }
    };
});

services.factory('database', function($platform, DB) {
    return {
        get: function(callback) {
            $platform.ready(function() {
                if (window.openDatabase) {
                    callback(new Database(window.openDatabase, DB));
                } else if (window.localStorage) {
                    callback(new LocalStorage(window.localStorage));
                } else {
                    callback(new InMemoryStorage({}));
                }
            });
        }
    };
});

services.service('settings', function($q, DatabaseService, database, Monitor, _) {
    var settings = new Settings({});
    this.get = function(callback) {
        database.get(function(db) {
            db.get('settings.search', function(search) {
                search = angular.fromJson(search);

                if (_.isUndefined(search.standort)) {
                    search.standort = true;
                }
                if (_.isUndefined(search.sortierung)) {
                    search.sortierung = _.first(Monitor.getSortierungen());
                }
                if (_.isUndefined(search.sorte)) {
                    search.sorte = _.first(Monitor.getSorten());
                }
                if (_.isUndefined(search.umkreis)) {
                    search.umkreis = _.first(Monitor.getUmkreise());
                }

                settings.setSearch(search);
                callback(settings);
            });
        });
    };
    this.update = function(search, successCallback, errorCallback) {
        database.get(function(db) {
            settings.setSearch(search);
            if (settings.isValid()) {
                db.save('settings.search', angular.toJson(settings.getSearch()), function() {
                    successCallback(settings);
                }, errorCallback);
            } else {
                if (errorCallback) {
                    errorCallback({error: 'Invalid'});
                }
            }
        });
    };
    this.getUser = function() {
        return DatabaseService.get('kasuser');
    };
    this.saveUser = function(user) {
        return DatabaseService.save('kasuser', user);
    };
});
